@@ -17,12 +17,12 @@ package Pod::Elemental::MakeSelector;
 # ABSTRACT: Build complex selectors as a single sub
 #---------------------------------------------------------------------
 
-use 5.010_001;                  # smart-matching is broken in 5.10.0
+use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
-# This file is part of Pod-Elemental-MakeSelector 0.04 (April 18, 2015)
+our $VERSION = '0.10';
+# This file is part of Pod-Elemental-MakeSelector 0.10 (May 30, 2015)
 
 use Carp qw(croak);
 
@@ -31,9 +31,6 @@ use Sub::Exporter -setup => {
   groups  => { default => [ qw(make_selector) ]},
 };
 
-# In Perl 5.18, smartmatch emits a warning
-no if $] >= 5.018, warnings => "experimental::smartmatch";
-
 #=====================================================================
 # Recturn true if the first element of the arrayref is not a string
 # starting with -
@@ -45,6 +42,16 @@ sub _has_optional_parameter
   @$inputR and (ref $inputR->[0] or not $inputR->[0] =~ /^-/);
 } # end _has_optional_parameter
 
+#---------------------------------------------------------------------
+sub required_parameter
+{
+  my ($inputR, $error_message) = @_;
+
+  croak($error_message) unless defined(my $val = shift @$inputR);
+
+  $val;
+} # end required_parameter
+
 #---------------------------------------------------------------------
 sub add_value
 {
@@ -65,6 +72,44 @@ sub join_expressions
   '(' . join("\n    $op ", @$expressionsR) . "\n  )";
 } # end join_expressions
 
+#---------------------------------------------------------------------
+# Supports only string, Regexp, or arrayref of either.
+# Nested arrayrefs should work, but are not documented.
+
+sub smart_match
+{
+  my ($valuesR, $value, $match) = @_;
+
+  TEST_REF: {
+    my $ref = ref $match;
+
+    if ($ref eq 'ARRAY') {
+      my $count = @$match;
+      if ($count == 0) {
+        return '0';               # Empty array never matches
+      } elsif ($count == 1) {
+        $match = $match->[0];
+        redo TEST_REF;
+      } else {
+        my $exp = join_expressions('or',
+          [ map smart_match($valuesR, '$v', $_), @$match ]
+        );
+        if ($value eq '$v') {
+          return $exp;
+        } else {
+          return sprintf 'do { my $v = %s; %s }', $value, $exp;
+        }
+      }
+    } elsif ($ref) {
+      return "$value =~ " . add_value($valuesR, $match);
+    } else {
+      return "$value eq " . add_value($valuesR, $match);
+    }
+  }
+
+  die "Can't reach";
+} # end smart_match
+
 #---------------------------------------------------------------------
 sub conjunction_action
 {
@@ -91,8 +136,8 @@ sub region_action
       if defined $pod;
 
   if (_has_optional_parameter($inputR)) {
-    my $name = add_value($valuesR, shift @$inputR);
-    push @expressions, "\$para->format_name ~~ $name";
+    push @expressions, smart_match($valuesR, '$para->format_name',
+                                   shift @$inputR);
   } # end if specific format(s) listed
 
   join_expressions(and => \@expressions);
@@ -118,7 +163,7 @@ our %action = (
     my ($valuesR, $inputR) = @_;
 
     my $name = add_value($valuesR,
-                          shift @$inputR // croak "-code requires a value");
+                         required_parameter($inputR, "-code requires a value"));
     "$name->(\$para)";
   }, #end -code
 
@@ -128,8 +173,7 @@ our %action = (
     my @expressions = type_action(qw(does Command));
 
     if (_has_optional_parameter($inputR)) {
-      my $name = add_value($valuesR, shift @$inputR);
-      push @expressions, "\$para->command ~~ $name";
+      push @expressions, smart_match($valuesR, '$para->command', shift @$inputR);
     } # end if specific command(s) listed
 
     join_expressions(and => \@expressions);
@@ -138,9 +182,8 @@ our %action = (
   -content => sub {
     my ($valuesR, $inputR) = @_;
 
-    my $name = add_value($valuesR,
-                          shift @$inputR // croak "-content requires a value");
-    "\$para->content ~~ $name";
+    smart_match($valuesR, '$para->content',
+                required_parameter($inputR, "-content requires a value"));
   }, #end -content
 
   -region       => \&region_action,
@@ -186,7 +229,7 @@ sub make_selector
                   $code)
       if @values;
 
-  #say $code;
+  #print STDERR $code;
   my ($sub, $err);
   {
     local $@;
@@ -221,8 +264,8 @@ Pod::Elemental::MakeSelector - Build complex selectors as a single sub
 
 =head1 VERSION
 
-This document describes version 0.04 of
-Pod::Elemental::MakeSelector, released April 18, 2015.
+This document describes version 0.10 of
+Pod::Elemental::MakeSelector, released May 30, 2015.
 
 =head1 SYNOPSIS
 
@@ -251,10 +294,13 @@ if the paragraph meets the selector's criteria.
 
 =head1 CRITERIA
 
-Most criteria that accept a parameter test it using smart matching,
-which means that they accept a string, a regex, or an arrayref of
-strings and/or regexes.  (This also means that Perl 5.10.1 is required
-to use Pod::Elemental::MakeSelector.)
+Most criteria that accept a parameter accept a string, a regex, or an
+arrayref of strings and/or regexes.  However,
+Pod::Elemental::MakeSelector I<does not> use Perl's C<~~> smartmatch
+operator because it is considered experimental.  Instoad, a limited
+form of smartmatching is performed by the code generator.  This means
+arrayrefs are iterated when the selector is compiled.  Modifying the
+arrayref later will not affect the selector.
 
 Optional parameters must not begin with C<->, or they will be treated
 as criteria instead.  If you need an optional parameter that begins
@@ -331,6 +377,8 @@ build_selector
 conjunction_action
 join_expressions
 region_action
+required_parameter
+smart_match
 type_action
 
 =head1 SEE ALSO
@@ -340,7 +388,7 @@ more limited than this module.
 
 =head1 DEPENDENCIES
 
-Pod::Elemental::MakeSelector requires L<Pod::Elemental> and Perl 5.10.1
+Pod::Elemental::MakeSelector requires L<Pod::Elemental> and Perl 5.8.0
 or later.
 
 =head1 BUGS
